---
title: llarp/rpc/rpc_request.hpp

---

# llarp/rpc/rpc_request.hpp



## Namespaces

| Name           |
| -------------- |
| **[llarp](Namespaces/namespacellarp.md)** <br>[crypto.hpp] |
| **[llarp::rpc](Namespaces/namespacellarp_1_1rpc.md)**  |




## Source code

```cpp
#pragma once

#include "json_bt.hpp"
#include <string_view>
#include <llarp/config/config.hpp>
#include <oxenmq/oxenmq.h>
#include <oxenmq/address.h>
#include <oxen/log/omq_logger.hpp>


namespace llarp::rpc {

  using nlohmann::json;

  using rpc_input = std::variant<nlohmann::json, oxenc::bt_dict_consumer>;

  template <typename RPC, typename RPCServer, typename RPCCallback>
  auto make_invoke() {
    return [](std::string_view body, RPCServer& server) -> typename RPCCallback::result_type {
      RPC rpc{};    // endpoint struct

      // parsing input as bt or json
      //    hand off to parse_request (overloaded versions)
      try {
          if (body->front() == 'd') { // Looks like a bt dict
            rpc.set_bt();
            parse_request(rpc, oxenc::bt_dict_consumer{*body});
          }
          else
            parse_request(rpc, body.empty() ? json::object() : json::parse(body));
      } catch (const std::exception& e) {
        throw parse_error{"Failed to parse request parameters: "s + e.what()};
      }

      server.invoke(rpc, std::move(request.context));

      if (rpc.response.is_null())
        rpc.response = json::object();
      
      if (rpc.is_bt())
        return json_to_bt(std::move(rpc.response));
      else
        return std::move(rpc.response);
    }
  }


} // namespace llarp::rpc
```


-------------------------------

Updated on 2023-01-13 at 22:44:39 +0000
