---
title: llarp/rpc/rpc_request_definitions.hpp

---

# llarp/rpc/rpc_request_definitions.hpp



## Namespaces

| Name           |
| -------------- |
| **[llarp](Namespaces/namespacellarp.md)** <br>[crypto.hpp] |
| **[llarp::rpc](Namespaces/namespacellarp_1_1rpc.md)**  |

## Classes

|                | Name           |
| -------------- | -------------- |
| struct | **[llarp::rpc::Halt](Classes/structllarp_1_1rpc_1_1Halt.md)**  |
| struct | **[llarp::rpc::Version](Classes/structllarp_1_1rpc_1_1Version.md)**  |
| struct | **[llarp::rpc::Status](Classes/structllarp_1_1rpc_1_1Status.md)**  |
| struct | **[llarp::rpc::GetStatus](Classes/structllarp_1_1rpc_1_1GetStatus.md)**  |
| struct | **[llarp::rpc::QuicConnect](Classes/structllarp_1_1rpc_1_1QuicConnect.md)**  |
| struct | **[llarp::rpc::QuicConnect::request_parameters](Classes/structllarp_1_1rpc_1_1QuicConnect_1_1request__parameters.md)**  |
| struct | **[llarp::rpc::QuicListener](Classes/structllarp_1_1rpc_1_1QuicListener.md)**  |
| struct | **[llarp::rpc::QuicListener::request_parameters](Classes/structllarp_1_1rpc_1_1QuicListener_1_1request__parameters.md)**  |
| struct | **[llarp::rpc::LookupSnode](Classes/structllarp_1_1rpc_1_1LookupSnode.md)**  |
| struct | **[llarp::rpc::LookupSnode::request_parameters](Classes/structllarp_1_1rpc_1_1LookupSnode_1_1request__parameters.md)**  |
| struct | **[llarp::rpc::Exit](Classes/structllarp_1_1rpc_1_1Exit.md)**  |
| struct | **[llarp::rpc::Exit::request_parameters](Classes/structllarp_1_1rpc_1_1Exit_1_1request__parameters.md)**  |
| struct | **[llarp::rpc::DNSQuery](Classes/structllarp_1_1rpc_1_1DNSQuery.md)**  |
| struct | **[llarp::rpc::DNSQuery::request_parameters](Classes/structllarp_1_1rpc_1_1DNSQuery_1_1request__parameters.md)**  |
| struct | **[llarp::rpc::Config](Classes/structllarp_1_1rpc_1_1Config.md)**  |
| struct | **[llarp::rpc::Config::request_parameters](Classes/structllarp_1_1rpc_1_1Config_1_1request__parameters.md)**  |




## Source code

```cpp
#pragma once

#include "json_binary_proxy.hpp"
#include "rpc_request_decorators.hpp"
#include <string_view>
#include <llarp/rpc/endpoints.hpp>
#include <llarp/config/config.hpp>
#include <oxenmq/oxenmq.h>
#include <oxenmq/address.h>
#include <oxen/log/omq_logger.hpp>



namespace llarp::rpc
{
  //  RPC: halt
  //    Stops lokinet router
  //
  //  Inputs: none
  //
  struct Halt : NoArgs, Immediate
  {
    static constexpr auto name = "halt"sv;
  };

  //  RPC: version
  //    Returns version and uptime information
  //
  //  Inputs: none
  //
  struct Version : NoArgs, Immediate
  {
    static constexpr auto name = "version"sv;
  };

  //  RPC: status
  //    Returns that current activity status of lokinet router
  //    Calls router::extractstatus
  //    
  //  Inputs: none
  //
  //  Returns:
  //    "running"
  //    "numNodesKnown"
  //    "dht"
  //    "services"
  //    "exit"
  //    "links"
  //    "outboundMessages"
  //
  struct Status : NoArgs, Deferred
  {
    static constexpr auto name = "status"sv;
  };

  //  RPC: get_status
  //    Returns current summary status
  //
  //  Inputs: none
  //
  //  Note: ask Jason
  //
  struct GetStatus : NoArgs, Deferred
  {
    static constexpr auto name = "get_status"sv;
  };

  //  RPC: quic_connect
  //    Initializes QUIC connection tunnel
  //    Passes request parameters in nlohmann::json format
  //
  //  Inputs:
  //    "endpoint" : endpoint id (string)
  //    "bind" : bind address (string, ex: "127.0.0.1:1142")
  //    "host" : remote host ID (string)
  //    "port" : port to bind to (int)
  //    "close" : close connection to port or host ID
  //
  //  Returns:
  //    "id" : connection ID
  //    "addr" : connection local address
  //    
  struct QuicConnect : Deferred
  {
    static constexpr auto name = "quic_connect"sv;

    struct request_parameters
    {
      std::string bindAddr;
      int closeID;
      std::string endpoint;
      uint16_t port;
      std::string remoteHost;
    } request;
  };

  //  RPC: quick_listener
  //    Connects to QUIC interface on local endpoint
  //    Passes request parameters in nlohmann::json format
  //
  //  Inputs:
  //    "endpoint" : endpoint id (string)
  //    "host" : remote host ID (string)
  //    "port" : port to bind to (int)
  //    "close" : close connection to port or host ID
  //    "srv-proto" : 
  //    
  //  Returns:
  //    "id" : connection ID
  //    "addr" : connection local address
  //    
  struct QuicListener : Deferred
  {
    static constexpr auto name = "quic_listener"sv;

    struct request_parameters
    {
      int closeID;
      std::string endpoint;
      uint16_t port;
      std::string remoteHost;
      std::string srvProto;
    } request;
  };

  //  RPC: lookup_snode
  //    Look up service node
  //    Passes request parameters in nlohmann::json format
  //
  //  Inputs:
  //    "snode" : router ID to query (string)
  //    
  //  Returns:
  //    "ip" : snode IP address
  //    
  struct LookupSnode : Deferred
  {
    static constexpr auto name = "lookup_snode"sv;

    struct request_parameters
    {
      std::string routerID;
    } request;
  };

  //  RPC: exit
  //    Seems like this adds an exit node?
  //    
  //  Note: ask Jason about the internals of this
  //  
  //  Inputs:
  //    ""
  //
  //  Returns:
  //    
  struct Exit : Deferred
  {
    static constexpr auto name = "exit"sv;

    struct request_parameters
    {
      std::string exitID;
    } request;
  };

  //  RPC: dns_query
  //    Attempts to query endpoint by domain name
  //
  //  Note: ask Jason about the internals of this
  //  
  //  Inputs:
  //    "endpoint" : endpoint ID to query (string)
  //    "qname" : query name (string)
  //    "qtype" : query type (int)
  //
  //  Returns:
  //    
  struct DNSQuery : Immediate
  {
    static constexpr auto name = "dns_query"sv;

    struct request_parameters
    {
      std::string endpoint;
      uint16_t qtype;
      std::string qname;
    } request;
  };

  //  RPC: config
  //    Runs lokinet router using .ini config file passed as path
  //
  //  Inputs: 
  //    "override" : path to config.ini file with which to override current config
  //    "reload" : restart router using current config file
  //    
  //  Returns:
  //    
  struct Config : Immediate
  {
    static constexpr auto name = "config"sv;

    struct request_parameters
    {
      std::string _override;
    } request;
  };

  // List of all RPC request structs to allow compile-time enumeration of all supported types
  using rpc_request_types = tools::type_list<
    Halt,
    Version,
    Status,
    GetStatus,
    QuicConnect,
    QuicListener,
    LookupSnode,
    Exit,
    DNSQuery,
    Config,
    >;

}  // namespace llarp::rpc


//  query oxenmq rpc ./lmq-rpc.py ipc://$HOME/.oxen/testnet/oxend.sock 'rpc.get_service_nodes' | jq .
//  
//  
```


-------------------------------

Updated on 2023-01-13 at 22:44:39 +0000
